﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Reflection;

namespace App6.Utils
{
    public class ReflectionHelper
    {
        #region Generics method invoker
        public static object InvokeGeneric(object target, string sMethod, Type type)
        {
            return InvokeGeneric(target, sMethod, new Type[] {
				type
			}, null);
        }

        public static object InvokeGeneric(object target, string sMethod, Type type,
                                           object parameter)
        {
            return InvokeGeneric(target, sMethod, new Type[] { type }, new object[] { parameter });
        }

        public static object InvokeGeneric(object target, string sMethod, Type type,
                                           object[] parameters)
        {
            return InvokeGeneric(target, sMethod, new Type[] { type }, parameters);
        }

        public static object InvokeGeneric(object target, string sMethod, Type[] type)
        {
            return InvokeGeneric(target, sMethod, type, null);
        }

        public static object InvokeGeneric(object target, string sMethod, Type[] type,
                                           object parameter)
        {
            return InvokeGeneric(target, sMethod, type, new object[] { parameter });
        }

        public static object InvokeGeneric(object target, string sMethod, Type[] type,
                                           object[] parameters)
        {
            MethodInfo method = target.GetType().GetMethod(sMethod);

            if (method == null)
            {
                System.Diagnostics.Debug.WriteLine("Method not found {0} in class {1}",
                                   sMethod, target.GetType().Name);

                return null;
            }

            MethodInfo methodGeneric = method.MakeGenericMethod(
                type);

            if (methodGeneric == null)
            {
                System.Diagnostics.Debug.WriteLine("Cannot convert method {0} in class {1} to required generic method.",
                                   sMethod, target.GetType().Name);

                return null;
            }

            return methodGeneric.Invoke(target, parameters);
        }

        #endregion

        #region Normal method invoker

        public static object Invoke(object target, string sMethod)
        {
            return Invoke(target, sMethod, null);
        }


        public static object Invoke(object target, string sMethod,
                                           object parameter)
        {
            return Invoke(target, sMethod, new object[] { parameter });
        }

        public static object Invoke(object target, string sMethod,
                                           object[] parameters)
        {
            try
            {
                MethodInfo method = target.GetType().GetMethod(sMethod);

                if (method == null)
                {
                    System.Diagnostics.Debug.WriteLine("Method not found {0} in class {1}",
                                       sMethod, target.GetType().Name);

                    return null;
                }

                return method.Invoke(target, parameters);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine("Method not found", ex);
            }

            return null;
        }


        #endregion

        #region Properties handling

        public static object GetProperty(object target, string name)
        {
            PropertyInfo pi = target.GetType().GetProperty(name);

            if (pi == null)
            {
                System.Diagnostics.Debug.WriteLine("Property not found {0} in class {1}",
                                   name, target.GetType().Name);

                return null;
            }

            return pi.GetValue(target, null);
        }

        public static void SetProperty(object target, string name, object val)
        {
            PropertyInfo pi = target.GetType().GetProperty(name);

            if (pi == null)
            {
                System.Diagnostics.Debug.WriteLine("Property not found {0} in class {1}",
                                   name, target.GetType().Name);
            }

            pi.SetValue(target, val, null);
        }

        #endregion

        #region Instance creation
        
        public static object CreateInstance(string classAndAssembly)
        {
            Type objType = Type.GetType(classAndAssembly);

            return Activator.CreateInstance(objType);
        }
        #endregion
    }
}